@@ -41,7 +41,7 @@ def requires(
             async def websocket_wrapper(
                 *args: typing.Any, **kwargs: typing.Any
             ) -> None:
-                websocket = kwargs.get("websocket", args[idx])
+                websocket = kwargs.get("websocket", args[idx] if args else None)
                 assert isinstance(websocket, WebSocket)
 
                 if not has_required_scope(websocket, scopes_list):
@@ -57,12 +57,14 @@ def requires(
             async def async_wrapper(
                 *args: typing.Any, **kwargs: typing.Any
             ) -> Response:
-                request = kwargs.get("request", args[idx])
+                request = kwargs.get("request", args[idx] if args else None)
                 assert isinstance(request, Request)
 
                 if not has_required_scope(request, scopes_list):
                     if redirect is not None:
-                        return RedirectResponse(url=request.url_for(redirect))
+                        return RedirectResponse(
+                            url=request.url_for(redirect), status_code=303
+                        )
                     raise HTTPException(status_code=status_code)
                 return await func(*args, **kwargs)
 
@@ -72,12 +74,14 @@ def requires(
             # Handle sync request/response functions.
             @functools.wraps(func)
             def sync_wrapper(*args: typing.Any, **kwargs: typing.Any) -> Response:
-                request = kwargs.get("request", args[idx])
+                request = kwargs.get("request", args[idx] if args else None)
                 assert isinstance(request, Request)
 
                 if not has_required_scope(request, scopes_list):
                     if redirect is not None:
-                        return RedirectResponse(url=request.url_for(redirect))
+                        return RedirectResponse(
+                            url=request.url_for(redirect), status_code=303
+                        )
                     raise HTTPException(status_code=status_code)
                 return func(*args, **kwargs)
 
@@ -94,7 +98,7 @@ class AuthenticationBackend:
     async def authenticate(
         self, conn: HTTPConnection
     ) -> typing.Optional[typing.Tuple["AuthCredentials", "BaseUser"]]:
-        raise NotImplemented()  # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
 
 class AuthCredentials:
@@ -105,15 +109,15 @@ class AuthCredentials:
 class BaseUser:
     @property
     def is_authenticated(self) -> bool:
-        raise NotImplemented()  # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
     @property
     def display_name(self) -> str:
-        raise NotImplemented()  # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
     @property
     def identity(self) -> str:
-        raise NotImplemented()  # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
 
 class SimpleUser(BaseUser):
