@@ -13,8 +13,8 @@ def build_environ(scope: Scope, body: bytes) -> dict:
     """
     environ = {
         "REQUEST_METHOD": scope["method"],
-        "SCRIPT_NAME": scope.get("root_path", ""),
-        "PATH_INFO": scope["path"],
+        "SCRIPT_NAME": scope.get("root_path", "").encode("utf8").decode("latin1"),
+        "PATH_INFO": scope["path"].encode("utf8").decode("latin1"),
         "QUERY_STRING": scope["query_string"].decode("ascii"),
         "SERVER_PROTOCOL": f"HTTP/{scope['http_version']}",
         "wsgi.version": (1, 0),
@@ -44,7 +44,8 @@ def build_environ(scope: Scope, body: bytes) -> dict:
             corrected_name = "CONTENT_TYPE"
         else:
             corrected_name = f"HTTP_{name}".upper().replace("-", "_")
-        # HTTPbis say only ASCII chars are allowed in headers, but we latin1 just in case
+        # HTTPbis say only ASCII chars are allowed in headers, but we latin1 just in
+        # case
         value = value.decode("latin1")
         if corrected_name in environ:
             value = environ[corrected_name] + "," + value
